Package: treeshap
Title: Compute SHAP Values for Your Tree-Based Models Using the
        'TreeSHAP' Algorithm
Version: 0.4.0
Authors@R: c(
    person("Michael", "Mayer", , "mayermichael79@gmail.com", role = c("aut", "cre")),
    person("Konrad", "Komisarczyk", email = "komisarczykkonrad@gmail.com", role = "aut"),
    person("Pawel", "Kozminski", email = "pkozminski99@gmail.com", role = "aut"),
    person("Szymon", "Maksymiuk", role = "aut", comment = c(ORCID = "0000-0002-3120-1601")),
    person("Lorenz A.", "Kapsner", role = "ctb", comment = c(ORCID = "0000-0003-1866-860X")),
    person("Mikolaj", "Spytek", role = "ctb", comment = c(ORCID = "0000-0001-7111-2286")),
    person("Mateusz", "Krzyzinski", email = "krzyzinskimateusz23@gmail.com", role = "ctb", comment = c(ORCID = "0000-0001-6143-488X")),
    person("Przemyslaw", "Biecek", email = "przemyslaw.biecek@gmail.com", role = c("aut", "cph"), comment = c(ORCID = "0000-0001-8423-1823"))
  )
Description: An efficient implementation of the 'TreeSHAP' algorithm
    introduced by Lundberg et al., (2020) <doi:10.1038/s42256-019-0138-9>.
    It is capable of calculating SHAP (SHapley Additive exPlanations) values 
    for tree-based models in polynomial time.  Currently supported models include 
    'gbm', 'randomForest', 'ranger', 'xgboost', 'lightgbm'.
License: GPL-3
URL: https://modeloriented.github.io/treeshap/,
        https://github.com/ModelOriented/treeshap
BugReports: https://github.com/ModelOriented/treeshap/issues
Depends: R (>= 4.1.0)
Imports: data.table, ggplot2, Rcpp
Suggests: gbm, jsonlite, lightgbm, gpboost, randomForest, ranger,
        scales, survival, testthat, xgboost
LinkingTo: Rcpp
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-04-24 07:49:57 UTC; mayer
Author: Michael Mayer [aut, cre],
  Konrad Komisarczyk [aut],
  Pawel Kozminski [aut],
  Szymon Maksymiuk [aut] (ORCID: <https://orcid.org/0000-0002-3120-1601>),
  Lorenz A. Kapsner [ctb] (ORCID:
    <https://orcid.org/0000-0003-1866-860X>),
  Mikolaj Spytek [ctb] (ORCID: <https://orcid.org/0000-0001-7111-2286>),
  Mateusz Krzyzinski [ctb] (ORCID:
    <https://orcid.org/0000-0001-6143-488X>),
  Przemyslaw Biecek [aut, cph] (ORCID:
    <https://orcid.org/0000-0001-8423-1823>)
Maintainer: Michael Mayer <mayermichael79@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-24 09:20:03 UTC
Built: R 4.4.3; x86_64-apple-darwin20; 2026-04-24 09:34:45 UTC; unix
Archs: treeshap.so.dSYM
