# Generated by roxygen2: do not edit by hand

S3method(predict,model_unified)
S3method(print,model_unified)
S3method(print,model_unified_multioutput)
S3method(print,treeshap)
S3method(print,treeshap_multioutput)
S3method(treeshap,model_unified)
S3method(treeshap,model_unified_multioutput)
S3method(unify,default)
S3method(unify,gbm)
S3method(unify,gpb.Booster)
S3method(unify,lgb.Booster)
S3method(unify,randomForest)
S3method(unify,ranger)
S3method(unify,xgb.Booster)
export(colors_breakdown_drwhy)
export(colors_discrete_drwhy)
export(gbm.unify)
export(gpboost.unify)
export(is.model_unified)
export(is.treeshap)
export(lightgbm.unify)
export(plot_contribution)
export(plot_feature_dependence)
export(plot_feature_importance)
export(plot_interaction)
export(randomForest.unify)
export(ranger.unify)
export(ranger_surv.unify)
export(set_reference_dataset)
export(theme_drwhy)
export(theme_drwhy_vertical)
export(treeshap)
export(unify)
export(xgboost.unify)
import(data.table)
import(ggplot2)
importFrom(Rcpp,sourceCpp)
importFrom(graphics,text)
importFrom(stats,reorder)
importFrom(stats,stepfun)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
useDynLib(treeshap)
