# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

predict_cpp <- function(x, is_na, roots, yes, no, missing, is_leaf, feature, split, decision_type, value) {
    .Call('_treeshap_predict_cpp', PACKAGE = 'treeshap', x, is_na, roots, yes, no, missing, is_leaf, feature, split, decision_type, value)
}

new_covers <- function(x, is_na, roots, yes, no, missing, is_leaf, feature, split, decision_type) {
    .Call('_treeshap_new_covers', PACKAGE = 'treeshap', x, is_na, roots, yes, no, missing, is_leaf, feature, split, decision_type)
}

treeshap_cpp <- function(x, is_na, roots, yes, no, missing, feature, split, decision_type, is_leaf, value, cover, verbose) {
    .Call('_treeshap_treeshap_cpp', PACKAGE = 'treeshap', x, is_na, roots, yes, no, missing, feature, split, decision_type, is_leaf, value, cover, verbose)
}

treeshap_interactions_cpp <- function(x, is_na, roots, yes, no, missing, feature, split, decision_type, is_leaf, value, cover, verbose) {
    .Call('_treeshap_treeshap_interactions_cpp', PACKAGE = 'treeshap', x, is_na, roots, yes, no, missing, feature, split, decision_type, is_leaf, value, cover, verbose)
}

