% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/treeshap.R
\name{print.treeshap}
\alias{print.treeshap}
\title{Prints treeshap objects}
\usage{
\method{print}{treeshap}(x, ...)
}
\arguments{
\item{x}{a treeshap object}

\item{...}{other arguments}
}
\value{
No return value, called for printing
}
\description{
Prints treeshap objects
}
